"""Benchmarks for assigning large NumPy data to protobuf fields."""

from collections.abc import Callable
import functools
import sys

import google_benchmark
import numpy as np

from google.protobuf import unittest_pb2


@functools.cache
def make_array(num_bytes: int):
  return np.zeros(shape=(num_bytes,), dtype=np.uint8)


def benchmark(
    func: Callable[[google_benchmark.State], None],
) -> Callable[[google_benchmark.State], None]:
  """Decorates a function for benchmarking."""

  @google_benchmark.register
  @google_benchmark.option.unit(google_benchmark.kMillisecond)
  @google_benchmark.option.arg_names(['num_bytes'])
  @google_benchmark.option.arg(1024 * 1024 * 20)
  @google_benchmark.option.arg(1024 * 1024 * 100)
  @functools.wraps(func)
  def wrapper(state: google_benchmark.State) -> None:
    func(state)
    state.bytes_processed = state.iterations * state.range(0)

  return wrapper


@benchmark
def bench_build_message_via_slice(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  while state:
    msg = unittest_pb2.TestAllTypes()
    msg.repeated_int32[:] = arr


@benchmark
def bench_build_message(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  while state:
    _ = unittest_pb2.TestAllTypes(repeated_int32=arr)


@benchmark
def bench_build_message_nested_via_slice(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  while state:
    msg = unittest_pb2.NestedTestAllTypes()
    msg.payload.repeated_int32[:] = arr


@benchmark
def bench_build_nested_message_dict(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  while state:
    _ = unittest_pb2.NestedTestAllTypes(payload=dict(repeated_int32=arr))


@benchmark
def bench_build_nested_message_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  while state:
    _ = unittest_pb2.NestedTestAllTypes(
        payload=unittest_pb2.TestAllTypes(repeated_int32=arr)
    )


@benchmark
def bench_build_nested_message_cord(state: google_benchmark.State):
  chunk_size = state.range(0) // 1000
  strings = ['a' * chunk_size] * 1000
  while state:
    _ = unittest_pb2.NestedTestAllTypes(
        payload=unittest_pb2.TestAllTypes(repeated_cord=strings)
    )


@benchmark
def bench_build_nested_message_string_piece(state: google_benchmark.State):
  chunk_size = state.range(0) // 1000
  strings = ['a' * chunk_size] * 1000
  while state:
    _ = unittest_pb2.NestedTestAllTypes(
        payload=unittest_pb2.TestAllTypes(repeated_string_piece=strings)
    )


@benchmark
def bench_build_nested_message_nested_message(state: google_benchmark.State):
  subs = [unittest_pb2.TestAllTypes.NestedMessage(bb=123)] * (
      state.range(0) // 8
  )
  while state:
    _ = unittest_pb2.NestedTestAllTypes(
        payload=unittest_pb2.TestAllTypes(repeated_nested_message=subs)
    )


@benchmark
def bench_assign_repeated_float(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.float32)
  msg = unittest_pb2.TestAllTypes()
  msg_source = unittest_pb2.TestAllTypes()
  msg_source.repeated_float.extend(arr)
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_float[:] = msg_source.repeated_float


@benchmark
def bench_assign_repeated_int64_to_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int64)
  msg = unittest_pb2.TestAllTypes()
  msg_source = unittest_pb2.TestAllTypes()
  msg_source.repeated_int64.extend(arr)
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_int32[:] = msg_source.repeated_int64


@benchmark
def bench_assign_repeated_double_to_float(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.float64)
  msg = unittest_pb2.TestAllTypes()
  msg_source = unittest_pb2.TestAllTypes()
  msg_source.repeated_double.extend(arr)
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_float[:] = msg_source.repeated_double


@benchmark
def bench_assign_numpy_int64_to_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int64)
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_int32[:] = arr


@benchmark
def bench_assign_numpy_double_to_float(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.float64)
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_float[:] = arr


@benchmark
def bench_extend_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_int32.extend(arr)


@benchmark
def bench_decode_from_bytes(state: google_benchmark.State):
  arr = make_array(state.range(0))
  msg = unittest_pb2.TestAllTypes()
  msg.optional_bytes = memoryview(arr)
  msg_bytes = msg.SerializeToString()
  while state:
    _ = unittest_pb2.TestAllTypes.FromString(msg_bytes)


@benchmark
def bench_decode_from_memoryview(state: google_benchmark.State):
  arr = make_array(state.range(0))
  msg = unittest_pb2.TestAllTypes()
  msg.optional_bytes = memoryview(arr)
  msg_bytes = memoryview(msg.SerializeToString())
  while state:
    _ = unittest_pb2.TestAllTypes.FromString(msg_bytes)


@benchmark
def bench_encode_into_bytes(state: google_benchmark.State):
  arr = make_array(state.range(0))
  msg = unittest_pb2.TestAllTypes()
  msg.optional_bytes = memoryview(arr)
  while state:
    _ = msg.SerializeToString()


@benchmark
def bench_assign_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  msg = unittest_pb2.TestAllTypes()
  while state:
    msg.repeated_int32[:] = arr
    state.pause_timing()
    msg.Clear()
    state.resume_timing()


@benchmark
def bench_assign_extend_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  msg = unittest_pb2.TestAllTypes()
  msg.repeated_int32[:] = arr
  while state:
    msg.repeated_int32[len(arr) :] = arr
    state.pause_timing()
    msg.repeated_int32[:] = arr
    state.resume_timing()


@benchmark
def bench_assign_slice_int32(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int32)
  half_arr = arr[: len(arr) // 2]
  msg = unittest_pb2.TestAllTypes()
  msg.repeated_int32.extend(arr)
  while state:
    msg.repeated_int32[::2] = half_arr


@benchmark
def bench_extend_int64(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.int64)
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_int64.extend(arr)


@benchmark
def bench_extend_float(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.float32)
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_float.extend(arr)


@benchmark
def bench_extend_double(state: google_benchmark.State):
  arr = make_array(state.range(0)).view(dtype=np.float64)
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.repeated_double.extend(arr)


@benchmark
def bench_assign_bytes(state: google_benchmark.State):
  arr = make_array(state.range(0))
  msg = unittest_pb2.TestAllTypes()
  arr_bytes = arr.tobytes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.optional_bytes = arr_bytes


@benchmark
def bench_assign_bytes_with_conversion(state: google_benchmark.State):
  arr = make_array(state.range(0))
  msg = unittest_pb2.TestAllTypes()
  while state:
    state.pause_timing()
    msg.Clear()
    state.resume_timing()
    msg.optional_bytes = memoryview(arr)


if __name__ == '__main__':
  if any(arg.startswith('--benchmark_filter') for arg in sys.argv):
    google_benchmark.main()
  else:
    print('No benchmark filter specified. Skipping benchmarks.')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/protobuf/json_options.proto
# Protobuf Python Version: 7.36.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    36,
    1,
    '',
    'google/protobuf/json_options.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import json_enumvalue_options_pb2 as google_dot_protobuf_dot_json__enumvalue__options__pb2

from google.protobuf.json_enumvalue_options_pb2 import *

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"google/protobuf/json_options.proto\x12\x02pb\x1a,google/protobuf/json_enumvalue_options.protoB\x1a\n\x18\x63om.google.protobuf.utilP\x00\x62\x08\x65\x64itionsp\xe9\x07')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.protobuf.json_options_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030com.google.protobuf.util'
# @@protoc_insertion_point(module_scope)



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd
"""Dynamic Protobuf class creator."""

from collections import OrderedDict
import hashlib
import os

from google.protobuf import descriptor
from google.protobuf import descriptor_pb2
from google.protobuf import descriptor_pool
from google.protobuf import message_factory


def _GetMessageFromFactory(pool, full_name):
  """Get a proto class from the MessageFactory by name.

  Args:
    pool: a descriptor pool.
    full_name: str, the fully qualified name of the proto type.

  Returns:
    A class, for the type identified by full_name.
  Raises:
    KeyError, if the proto is not found in the factory's descriptor pool.
  """
  proto_descriptor = pool.FindMessageTypeByName(full_name)
  proto_cls = message_factory.GetMessageClass(proto_descriptor)
  return proto_cls


def MakeSimpleProtoClass(fields, full_name=None, pool=None):
  """Create a Protobuf class whose fields are basic types.

  Note: this doesn't validate field names!

  Args:
    fields: dict of {name: field_type} mappings for each field in the proto. If
      this is an OrderedDict the order will be maintained, otherwise the fields
      will be sorted by name.
    full_name: optional str, the fully-qualified name of the proto type.
    pool: optional DescriptorPool instance.

  Returns:
    a class, the new protobuf class with a FileDescriptor.
  """
  pool_instance = pool or descriptor_pool.DescriptorPool()
  if full_name is not None:
    try:
      proto_cls = _GetMessageFromFactory(pool_instance, full_name)
      return proto_cls
    except KeyError:
      # The factory's DescriptorPool doesn't know about this class yet.
      pass

  # Get a list of (name, field_type) tuples from the fields dict. If fields was
  # an OrderedDict we keep the order, but otherwise we sort the field to ensure
  # consistent ordering.
  field_items = fields.items()
  if not isinstance(fields, OrderedDict):
    field_items = sorted(field_items)

  # Use a consistent file name that is unlikely to conflict with any imported
  # proto files.
  fields_hash = hashlib.sha1(usedforsecurity=False)
  for f_name, f_type in field_items:
    fields_hash.update(f_name.encode('utf-8'))
    fields_hash.update(str(f_type).encode('utf-8'))
  proto_file_name = fields_hash.hexdigest() + '.proto'

  # If the proto is anonymous, use the same hash to name it.
  if full_name is None:
    full_name = (
        'net.proto2.python.public.proto_builder.AnonymousProto_'
        + fields_hash.hexdigest()
    )
    try:
      proto_cls = _GetMessageFromFactory(pool_instance, full_name)
      return proto_cls
    except KeyError:
      # The factory's DescriptorPool doesn't know about this class yet.
      pass

  # This is the first time we see this proto: add a new descriptor to the pool.
  pool_instance.Add(
      _MakeFileDescriptorProto(proto_file_name, full_name, field_items)
  )
  return _GetMessageFromFactory(pool_instance, full_name)


def _MakeFileDescriptorProto(proto_file_name, full_name, field_items):
  """Populate FileDescriptorProto for MessageFactory's DescriptorPool."""
  package, name = full_name.rsplit('.', 1)
  file_proto = descriptor_pb2.FileDescriptorProto()
  file_proto.name = os.path.join(package.replace('.', '/'), proto_file_name)
  file_proto.package = package
  desc_proto = file_proto.message_type.add()
  desc_proto.name = name
  for f_number, (f_name, f_type) in enumerate(field_items, 1):
    field_proto = desc_proto.field.add()
    field_proto.name = f_name
    # # If the number falls in the reserved range, reassign it to the correct
    # # number after the range.
    if f_number >= descriptor.FieldDescriptor.FIRST_RESERVED_FIELD_NUMBER:
      f_number += (
          descriptor.FieldDescriptor.LAST_RESERVED_FIELD_NUMBER
          - descriptor.FieldDescriptor.FIRST_RESERVED_FIELD_NUMBER
          + 1
      )
    field_proto.number = f_number
    field_proto.label = descriptor_pb2.FieldDescriptorProto.LABEL_OPTIONAL
    field_proto.type = f_type
  return file_proto



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd
"""Contains Unknown Fields APIs.

Simple usage example:
  unknown_field_set = UnknownFieldSet(message)
  for unknown_field in unknown_field_set:
    wire_type = unknown_field.wire_type
    field_number = unknown_field.field_number
    data = unknown_field.data
"""

from google.protobuf.internal import api_implementation

if api_implementation._c_module is not None:  # pylint: disable=protected-access
  UnknownFieldSet = api_implementation._c_module.UnknownFieldSet  # pylint: disable=protected-access
else:
  from google.protobuf.internal import decoder  # pylint: disable=g-import-not-at-top
  from google.protobuf.internal import wire_format  # pylint: disable=g-import-not-at-top

  class UnknownField:
    """A parsed unknown field."""

    # Disallows assignment to other attributes.
    __slots__ = ['_field_number', '_wire_type', '_data']

    def __init__(self, field_number, wire_type, data):
      self._field_number = field_number
      self._wire_type = wire_type
      self._data = data
      return

    @property
    def field_number(self):
      return self._field_number

    @property
    def wire_type(self):
      return self._wire_type

    @property
    def data(self):
      return self._data

  class UnknownFieldSet:
    """UnknownField container."""

    # Disallows assignment to other attributes.
    __slots__ = ['_values']

    def __init__(self, msg):

      def InternalAdd(field_number, wire_type, data):
        unknown_field = UnknownField(field_number, wire_type, data)
        self._values.append(unknown_field)

      self._values = []
      msg_des = msg.DESCRIPTOR
      # pylint: disable=protected-access
      unknown_fields = msg._unknown_fields
      if msg_des.has_options and msg_des.GetOptions().message_set_wire_format:
        local_decoder = decoder.UnknownMessageSetItemDecoder()
        for _, buffer in unknown_fields:
          field_number, data = local_decoder(memoryview(buffer))
          InternalAdd(field_number, wire_format.WIRETYPE_LENGTH_DELIMITED, data)
      else:
        for tag_bytes, buffer in unknown_fields:
          field_number, wire_type = decoder.DecodeTag(tag_bytes)
          if field_number == 0:
            raise RuntimeError('Field number 0 is illegal.')
          data, _ = decoder._DecodeUnknownField(
              memoryview(buffer), 0, len(buffer), field_number, wire_type
          )
          InternalAdd(field_number, wire_type, data)

    def __getitem__(self, index):
      size = len(self._values)
      if index < 0:
        index += size
      if index < 0 or index >= size:
        raise IndexError('index %d out of range'.index)

      return self._values[index]

    def __len__(self):
      return len(self._values)

    def __iter__(self):
      return iter(self._values)



# Protocol Buffers - Google's data interchange format
# Copyright 2026 Google LLC.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Microbenchmarks for JSON format parsing and custom enum names."""

import statistics
import timeit
import unittest

from google.protobuf import json_format

from google.protobuf.json import json_enumval_custom_string_pb2


class JsonFormatBenchmark(unittest.TestCase):
  """Microbenchmarks for JSON format parsing."""

  def test_benchmark_parse_json(self):
    """Benchmarks parsing of default, custom, and unknown enum names in JSON."""
    default_payloads = [
        '{"armor":"ARMOR_GREAT_HELM"}',
        '{"armor":"ARMOR_GAUNTLET"}',
        '{"armor":"ARMOR_PLATE"}',
        '{"armor":"ARMOR_COIF"}',
        '{"armor":"ARMOR_PAULDRON"}',
        '{"armor":"ARMOR_SABATON"}',
        '{"armor":"ARMOR_HACHI_MAI_DO"}',
    ]
    custom_payloads = [
        '{"armor":"gr8 helm"}',
        '{"armor":"a\\"b"}',
        '{"armor":"\\"plate\\""}',
        '{"armor":""}',
        '{"armor":"p\\taul\\ndron"}',
        '{"armor":"sabaton"}',
        '{"armor":"8"}',
    ]
    unknown_payloads = [
        '{"armor":"UNKNOWN_1"}',
        '{"armor":"UNKNOWN_2"}',
        '{"armor":"UNKNOWN_3"}',
        '{"armor":"UNKNOWN_4"}',
        '{"armor":"UNKNOWN_5"}',
        '{"armor":"UNKNOWN_6"}',
        '{"armor":"UNKNOWN_7"}',
    ]

    repeated_default_payload = (
        '{"armors":['
        + ','.join(
            [
                '"ARMOR_GREAT_HELM"',
                '"ARMOR_GAUNTLET"',
                '"ARMOR_PLATE"',
                '"ARMOR_COIF"',
                '"ARMOR_PAULDRON"',
                '"ARMOR_SABATON"',
                '"ARMOR_HACHI_MAI_DO"',
            ]
            * 14
        )
        + ']}'
    )
    repeated_custom_payload = (
        '{"armors":['
        + ','.join(
            [
                '"gr8 helm"',
                '"a\\"b"',
                '"\\"plate\\""',
                '""',
                '"p\\taul\\ndron"',
                '"sabaton"',
                '"8"',
            ]
            * 14
        )
        + ']}'
    )
    repeated_unknown_payload = (
        '{"armors":['
        + ','.join(
            [
                '"UNKNOWN_1"',
                '"UNKNOWN_2"',
                '"UNKNOWN_3"',
                '"UNKNOWN_4"',
                '"UNKNOWN_5"',
                '"UNKNOWN_6"',
                '"UNKNOWN_7"',
            ]
            * 14
        )
        + ']}'
    )

    iterations = 10
    trials = 100
    num_parses = 10
    num_items = 98

    msg = json_enumval_custom_string_pb2.Knight()

    # Warmup phase to eliminate initial compilation/import overhead
    for _ in range(200):
      for p in custom_payloads:
        msg.Clear()
        json_format.Parse(p, msg)

    def run_def():
      for p in default_payloads:
        for _ in range(num_parses):
          msg.Clear()
          json_format.Parse(p, msg)

    def run_cust():
      for p in custom_payloads:
        for _ in range(num_parses):
          msg.Clear()
          json_format.Parse(p, msg)

    def run_unk():
      for p in unknown_payloads:
        for _ in range(num_parses):
          msg.Clear()
          json_format.Parse(p, msg, ignore_unknown_fields=True)

    def run_rep_def():
      for _ in range(num_parses):
        msg.Clear()
        json_format.Parse(repeated_default_payload, msg)

    def run_rep_cust():
      for _ in range(num_parses):
        msg.Clear()
        json_format.Parse(repeated_custom_payload, msg)

    def run_rep_unk():
      for _ in range(num_parses):
        msg.Clear()
        json_format.Parse(
            repeated_unknown_payload, msg, ignore_unknown_fields=True
        )

    benchmarks = [
        (
            'run_def',
            timeit.Timer(run_def, setup=msg.Clear),
            len(default_payloads) * num_parses,
        ),
        (
            'run_cust',
            timeit.Timer(run_cust, setup=msg.Clear),
            len(custom_payloads) * num_parses,
        ),
        (
            'run_unk',
            timeit.Timer(run_unk, setup=msg.Clear),
            len(unknown_payloads) * num_parses,
        ),
        (
            'run_rep_def',
            timeit.Timer(run_rep_def, setup=msg.Clear),
            num_items * num_parses,
        ),
        (
            'run_rep_cust',
            timeit.Timer(run_rep_cust, setup=msg.Clear),
            num_items * num_parses,
        ),
        (
            'run_rep_unk',
            timeit.Timer(run_rep_unk, setup=msg.Clear),
            num_items * num_parses,
        ),
    ]

    # Interleaved trials to minimize CPU frequency scaling / ordering bias
    benchmark_samples = {name: [] for name, _, _ in benchmarks}
    for _ in range(trials):
      for name, timer, num_ops_per_batch in benchmarks:
        total_ops = iterations * num_ops_per_batch
        elapsed = timer.timeit(number=iterations)
        benchmark_samples[name].append(elapsed * 1e6 / total_ops)

    def stats(samples):
      if len(samples) < 100:
        raise ValueError(
            f'Insufficient samples ({len(samples)}) for statistical'
            ' calculations. At least 100 samples are required.'
        )
      med_val = statistics.median(samples)
      mean_val = statistics.mean(samples)
      stdev_val = statistics.stdev(samples)
      p99_val = statistics.quantiles(samples, n=100)[98]
      return p99_val, med_val, mean_val, stdev_val

    p99_def, med_def, mean_def, std_def = stats(benchmark_samples['run_def'])
    p99_cust, med_cust, mean_cust, std_cust = stats(
        benchmark_samples['run_cust']
    )
    p99_unk, med_unk, mean_unk, std_unk = stats(benchmark_samples['run_unk'])
    p99_rep_def, med_rep_def, mean_rep_def, std_rep_def = stats(
        benchmark_samples['run_rep_def']
    )
    p99_rep_cust, med_rep_cust, mean_rep_cust, std_rep_cust = stats(
        benchmark_samples['run_rep_cust']
    )
    p99_rep_unk, med_rep_unk, mean_rep_unk, std_rep_unk = stats(
        benchmark_samples['run_rep_unk']
    )

    print(
        f'\n[BENCHMARK] ParseJsonDefault:                med {med_def:.2f}'
        f' us/op | p99 {p99_def:.2f} us/op | mean {mean_def:.2f} ±'
        f' {std_def:.2f} us/op'
    )
    print(
        f'[BENCHMARK] ParseJsonCustom:                 med {med_cust:.2f} us/op'
        f' | p99 {p99_cust:.2f} us/op | mean {mean_cust:.2f} ± {std_cust:.2f}'
        ' us/op'
    )
    print(
        '[BENCHMARK] ParseJsonUnknownIgnored:        '
        f' med {med_unk:.2f} us/op | p99 {p99_unk:.2f} us/op | mean'
        f' {mean_unk:.2f} ± {std_unk:.2f} us/op'
    )
    print(
        '[BENCHMARK] ParseRepeatedJsonDefault:       '
        f' med {med_rep_def:.2f} us/item | p99 {p99_rep_def:.2f} us/item |'
        f' mean {mean_rep_def:.2f} ± {std_rep_def:.2f} us/item'
    )
    print(
        '[BENCHMARK] ParseRepeatedJsonCustom:        '
        f' med {med_rep_cust:.2f} us/item | p99 {p99_rep_cust:.2f} us/item |'
        f' mean {mean_rep_cust:.2f} ± {std_rep_cust:.2f} us/item'
    )
    print(
        '[BENCHMARK] ParseRepeatedJsonUnknownIgnored:'
        f' med {med_rep_unk:.2f} us/item | p99 {p99_rep_unk:.2f} us/item |'
        f' mean {mean_rep_unk:.2f} ± {std_rep_unk:.2f} us/item'
    )


if __name__ == '__main__':
  unittest.main()
